# IMPORT
import lightgbm as lgb
from sklearn.svm import SVC
from xgboost import XGBClassifier
from aiautomation.mlpackage.Entities import MultiClasModelEnitity
from catboost import CatBoostClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.neighbors import KNeighborsClassifier
from aiautomation.mlpackage.PackageVariable import Variable
from sklearn.linear_model import LogisticRegression, RidgeClassifier
from sklearn.multiclass import OneVsOneClassifier
from sklearn.ensemble import RandomForestClassifier, GradientBoostingClassifier, AdaBoostClassifier


class Models:

    def __init__(self):
        self.alg_name = {}

    # define different models
    # LOGISTIC REGRESSION
    def log_reg(self):
        # define models and parameters
        # TODO max_iteration =1000
        logistic_reg = LogisticRegression(max_iter=1100)

        # Whole values
        solvers = ['newton-cg', 'lbfgs', 'liblinear', 'sag', 'saga']
        penalty = ['l2']
        c_values = [100, 10, 1.0, 0.1, 0.01]

        # define grid search
        grid = dict(estimator__solver=solvers, estimator__penalty=penalty, estimator__C=c_values)

        # multi class classification
        ovo = OneVsOneClassifier(logistic_reg)

        # Start the model to run
        self.alg_name[logistic_reg] = 'sklearn.linear_model.LogisticRegression'

        gene_grid = dict(solver=solvers, penalty=penalty, C=c_values)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeLogReg, gene_grid, logistic_reg)
        return multi_class_model_entity

    # -----------------------------------------------------------------------------
    # RIDGE CLASSIFIER
    def rid_clas(self):
        # define models and parameters
        rid_clas = RidgeClassifier()

        # Whole values
        alpha = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]

        # define grid search
        grid = dict(estimator__alpha=alpha)

        # multi class classification
        ovo = OneVsOneClassifier(rid_clas)

        # Start the model to run
        self.alg_name[rid_clas] = 'sklearn.linear_model.RidgeClassifier'

        gene_grid = dict(alpha=alpha)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeRidge, gene_grid, rid_clas)
        return multi_class_model_entity

    # -----------------------------------------------------------------------------
    # K NEAREST NEIGHBOUR
    def knn(self):
        # define models and parameters
        knn = KNeighborsClassifier()

        # Whole values
        n_neighbors = list(range(1, 21, 2))
        weights = ['uniform', 'distance']
        metric = ['euclidean', 'manhattan', 'minkowski']

        # define grid search
        grid = dict(estimator__n_neighbors=n_neighbors, estimator__weights=weights, estimator__metric=metric)

        # multi class classification
        ovo = OneVsOneClassifier(knn)

        # Start the model to run
        self.alg_name[knn] = 'sklearn.neighbors.KNeighborsClassifier'

        gene_grid = dict(n_neighbors=n_neighbors, weights=weights, metric=metric)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeKnn, gene_grid, knn)
        return multi_class_model_entity

    # -----------------------------------------------------------------------------
    # Support Vector Machine (SVM)
    def svm(self):
        # define models and parameters
        svc_classifier = SVC()

        # Whole values
        kernel = ['poly', 'rbf', 'sigmoid']
        c = [100, 10, 1.0, 0.1, 0.01]
        gamma = ['scale']

        # define grid search
        grid = dict(estimator__kernel=kernel, estimator__C=c, estimator__gamma=gamma)

        # multi class classification
        ovo = OneVsOneClassifier(svc_classifier)

        # Start the model to run	
        self.alg_name[svc_classifier] = 'sklearn.svm.SVC'

        gene_grid = dict(kernel=kernel, C=c, gamma=gamma)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeSvc, gene_grid, svc_classifier)
        return multi_class_model_entity

    # -----------------------------------------------------------------------------
    # DECISION TREE
    def des_tree(self):
        # define models and parameters
        des_tree_classifier = DecisionTreeClassifier()

        # Whole values
        max_depth = [3, None]
        max_features = list(range(1, 10, 1))
        min_sample_leaf = list(range(1, 10, 1))
        criterion = ['gini', 'entropy']

        # define grid search
        grid = dict(estimator__max_depth=max_depth, estimator__max_features=max_features,
                    estimator__min_samples_leaf=min_sample_leaf, estimator__criterion=criterion)

        # multi class classification
        ovo = OneVsOneClassifier(des_tree_classifier)

        # Start the model to run
        self.alg_name[des_tree_classifier] = 'sklearn.tree.DecisionTreeClassifier'

        gene_grid = dict(max_depth=max_depth, max_features=max_features, min_samples_leaf=min_sample_leaf,
                         criterion=criterion)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeDesTree, gene_grid,
                                                         des_tree_classifier)
        return multi_class_model_entity

    # -----------------------------------------------------------------------------
    # RANDOM FOREST
    def ran_for(self):
        # define models and parameters
        ran_for_classifier = RandomForestClassifier()

        # Whole values
        n_estimators = list(range(100, 1001, 100))
        max_features = ['sqrt', 'log2']

        # define grid search
        grid = dict(estimator__n_estimators=n_estimators, estimator__max_features=max_features)

        # multi class classification
        ovo = OneVsOneClassifier(ran_for_classifier)

        # Start the model to runs
        self.alg_name[ran_for_classifier] = 'sklearn.ensemble.RandomForestClassifier'

        gene_grid = dict(n_estimators=n_estimators, max_features=max_features)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeRanFor, gene_grid, ran_for_classifier)
        return multi_class_model_entity

    '''#-----------------------------------------------------------------------------
    #GAUSSIAN NB
    def gaussianNB():
        start = time.time()
        print ("GAUSSIAN NAIVE BAYES START  ------------")

        ## define models and parameters
        gaussian_nb = GaussianNB()
        var_smoothing = [1e-9, 1e-8, 1e-7, 1e-6, 1e-5, 1e-4, 1e-3, 1e-2, 1e-1, 1e-0, 1]

        #define grid search
        grid = dict(var_smoothing = var_smoothing)

         #Start the model alg_name[gaussian_nb] = 'sklearn.naive_bayes.GaussianNB' t11 = threading.Thread(
         target=model_grid_fit,args=(gaussian_nb, X, Y,X_test,grid,"GAUSSIAN_NB_submission_grid")) t12 = 
         threading.Thread(target=model_bayes_fit,args = (gaussian_nb, X, Y,X_test,grid, 
         "GAUSSIAN_NB_submission_bayes")) t13 = threading.Thread(target=model_gene_fit,args = (gaussian_nb, X, Y,
         X_test,grid, "GAUSSIAN_NB_submission_gene")) t14 = threading.Thread(target=model_optuna_fit,
         args = (gaussian_nb, X, Y, X_test,grid,"GAUSSIAN_NB_submission_optuna")) t11.start() t12.start() t13.start() 
         t14.start() t11.join() t12.join() t13.join() t14.join() 

        end = time.time()	
        print ("Gaussian Naive bayes Finish. In time of seconds = ",int(end-start),"\n\n\n")


    #-----------------------------------------------------------------------------
    #MULTINOMIAL NB
    def multinomialNB():
        start = time.time()
        print ("Multinomial NAIVE BAYES START  ------------")

        ## define models and parameters
        multinomial_nb = MultinomialNB()
        alpha = [0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0]
        fit_prior = [False, True]

        # define grid search
        grid = dict(alpha = alpha, fit_prior = fit_prior)

         #Start the model alg_name[multinomial_nb] = 'sklearn.naive_bayes.MultinomialNB' t11 = threading.Thread(
         target=model_grid_fit,args=(multinomial_nb, X, Y,X_test,grid,"MULTINOMIAL_NB_submission_grid")) t12 = 
         threading.Thread(target=model_bayes_fit,args = (multinomial_nb, X, Y,X_test,grid,
         "MULTINOMIAL_NB_submission_bayes")) t13 = threading.Thread(target=model_gene_fit,args = (multinomial_nb, X, Y,
         X_test,grid,"MULTINOMIAL_NB_submission_gene")) t14 = threading.Thread(target=model_optuna_fit,
         args = (multinomial_nb, X, Y, X_test,grid,"MULTINOMIAL_NB_submission_optuna")) t11.start() t12.start() 
         t13.start() t14.start() t11.join() t12.join() t13.join() t14.join() 

        end = time.time()	
        print ("Multinomial Naive bayes Finish. In time of seconds = ",int(end-start),"\n\n\n")'''

    # -----------------------------------------------------------------------------
    # Stochastic Gradient Boosting
    def grad_boost(self):
        # define models and parameters
        grad_boost_class = GradientBoostingClassifier()

        # Whole values
        n_estimators = list(range(100, 1000, 100))
        learning_rate = [0.001, 0.01, 0.1]
        subsample = [0.5, 0.7, 1.0]
        max_depth = [3, 7, 9]

        # define grid search
        grid = dict(estimator__learning_rate=learning_rate, estimator__n_estimators=n_estimators,
                    estimator__subsample=subsample, estimator__max_depth=max_depth)

        # multi class classification
        ovo = OneVsOneClassifier(grad_boost_class)

        # Start the model
        self.alg_name[grad_boost_class] = 'sklearn.ensemble.GradientBoostingClassifier'

        gene_grid = dict(learning_rate=learning_rate, n_estimators=n_estimators, subsample=subsample,
                         max_depth=max_depth)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeGradBoost, gene_grid, grad_boost_class)
        return multi_class_model_entity

    # -----------------------------------------------------------------------------
    # Adaptive Boosting
    def ada_boost(self):
        """
        max_depth = [3,None]
        max_features = [1,2,3,4,5,6,7,8,9,10]
        min_sample_leaf = [1,2,3,4,5,6,7,8,9,10]
        criterion = ['gini','entropy']

        des_classifier = DecisionTreeClassifier(max_depth=max_depth,max_features=max_features,
        min_samples_leaf=min_sample_leaf,criterion=criterion) """

        # define models and parameters
        ada_boost_class = AdaBoostClassifier()
        n_estimators = list(range(50, 1001, 50))
        learning_rate = [0.001, 0.01, 0.1]
        algorithm = ['SAMME', 'SAMME.R']

        # define grid search
        grid = dict(estimator__learning_rate=learning_rate, estimator__n_estimators=n_estimators,
                    estimator__algorithm=algorithm)

        # multi class classification
        ovo = OneVsOneClassifier(ada_boost_class)

        # Start the model
        self.alg_name[ada_boost_class] = 'sklearn.ensemble.AdaBoostClassifier'

        # define grid search
        gene_grid = dict(learning_rate=learning_rate, n_estimators=n_estimators, algorithm=algorithm)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeAdaBoost, gene_grid, ada_boost_class)
        return multi_class_model_entity

    # -----------------------------------------------------------------------------
    # XGB boost
    def xgb(self):
        # define models and parameters
        xgb_model = XGBClassifier()

        # Whole values
        max_depth = list(range(3, 10, 2))
        min_child_weight = list(range(1, 6, 2))
        gamma = [float(i / 10.0) for i in range(0, 5)]
        n_estimators = list(range(100, 1000, 100))
        learning_rate = [0.001, 0.01, 0.1]
        subsample = [i / 10.0 for i in range(6, 10)],
        col_sample_by_tree = [i / 10.0 for i in range(6, 10)]
        reg_alpha = [1e-5, 1e-2, 0.1, 1, 100, 0, 0.001, 0.005, 0.01, 0.05]

        # define grid search
        grid = dict(estimator__learning_rate=learning_rate, estimator__n_estimators=n_estimators,
                    estimator__subsample=subsample, estimator__max_depth=max_depth,
                    estimator__min_child_weight=min_child_weight, estimator__gamma=gamma,
                    estimator__colsample_bytree=col_sample_by_tree, estimator__reg_alpha=reg_alpha)

        # multi class classification
        ovo = OneVsOneClassifier(xgb_model)

        # Start the model
        self.alg_name[xgb_model] = 'xgboost.XGBClassifier'

        gene_grid = dict(learning_rate=learning_rate, n_estimators=n_estimators, subsample=subsample,
                         max_depth=max_depth, min_child_weight=min_child_weight, gamma=gamma,
                         colsample_bytree=col_sample_by_tree, reg_alpha=reg_alpha)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeXgb, gene_grid, xgb_model)
        return multi_class_model_entity

    # -----------------------------------------------------------------------------
    # Light GBM
    def light_gbm(self):
        # define models and parameters
        lightgbm = lgb.LGBMClassifier()

        # Whole values
        num_leaves = list(range(50, 1000, 50))
        learning_rate = [0.001, 0.01, 0.1]
        max_depth = list(range(3, 10, 2))
        max_bin = list(range(100, 1000, 100))
        objective = 'binary'

        # define grid search
        grid = dict(estimator__learning_rate=learning_rate, estimator__num_leaves=num_leaves,
                    estimator__max_bin=max_bin, estimator__max_depth=max_depth, estimator__objective=objective)

        # multi class classification
        ovo = OneVsOneClassifier(lightgbm)

        # Start the model
        self.alg_name[lightgbm] = 'lgb.LGBMClassifier'

        gene_grid = dict(learning_rate=learning_rate, num_leaves=num_leaves, max_bin=max_bin, max_depth=max_depth,
                         objective=objective)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeLGBM, gene_grid, lightgbm)
        return multi_class_model_entity

    # -----------------------------------------------------------------------------
    # Cat Boost GBM
    @staticmethod
    def cat_boost_gbm():
        # define models and parameters
        catboost = CatBoostClassifier()

        # Whole values
        depth = [3, 1, 2, 6, 4, 5, 7, 8, 9, 10]
        iterations = [250, 100, 500, 1000]
        learning_rate = [0.03, 0.001, 0.01, 0.1, 0.2, 0.3]
        l2_leaf_reg = [3, 1, 5, 10, 100]
        border_count = [32, 5, 10, 20, 50, 100, 200],
        ctr_border_count = [50, 5, 10, 20, 100, 200],
        thread_count = 4

        # define grid search
        grid = dict(estimator__learning_rate=learning_rate, estimator__depth=depth, estimator__iterations=iterations,
                    estimator__l2_leaf_reg=l2_leaf_reg, estimator__border_count=border_count,
                    estimator__ctr_border_count=ctr_border_count, estimator__thread_count=thread_count)

        # multi class classification
        ovo = OneVsOneClassifier(catboost)

        # Start the model
        # self.alg_name[catboost] = 'catboost.CatBoostClassifier'

        gene_grid = dict(learning_rate=learning_rate, depth=depth, iterations=iterations, l2_leaf_reg=l2_leaf_reg,
                         border_count=border_count, ctr_border_count=ctr_border_count, thread_count=thread_count)

        multi_class_model_entity = MultiClasModelEnitity(ovo, grid, Variable.typeCGBM, gene_grid, catboost)
        return multi_class_model_entity

    def get_all_models(self):
        return [self.rid_clas(), self.log_reg(), self.knn(), self.des_tree(),
                self.svm()]  # , self.ran_for(), self.ada_Boost(), self.grad_boost(), self.xgb(), self.light_gbm(),
        # self.cat_boost_gbm()]

    def get_algorithm_name(self):
        return self.alg_name



# IMPORT
import pandas as pd
from imblearn.over_sampling import SMOTE
from imblearn.over_sampling import RandomOverSampler
from sklearn.preprocessing import LabelEncoder, RobustScaler, MinMaxScaler


class DataConversion:

    # Functions used in this project
    @staticmethod
    def convert_to_num(dataset):
        cat_data = []
        num_data = []

        for i, c in enumerate(dataset.dtypes):
            if c == object:
                cat_data.append(dataset.iloc[:, i])
            else:
                num_data.append(dataset.iloc[:, i])

        cat_data = pd.DataFrame(cat_data).transpose()
        num_data = pd.DataFrame(num_data).transpose()

        # cat_data
        # If you want to fill every column with its own most frequent value you can use
        cat_data = cat_data.apply(lambda x: x.fillna(x.value_counts().index[0]))

        # num_data
        # fill every missing value with their previous value in the same column
        # num_data.fillna(method='bfill', inplace=True)

        num_data = num_data.apply(lambda x: x.fillna(x.mean()), axis=0)

        # Transform categorical data
        le = LabelEncoder()
        for i in cat_data:
            cat_data[i] = le.fit_transform(cat_data[i])

        df = pd.concat([cat_data, num_data], axis=1)
        return df

    @staticmethod
    def normalize_data(x):
        # Normalize numerical data
        # fit scaler on training data
        min_max_scaler = MinMaxScaler()
        x_scaled = min_max_scaler.fit_transform(x)
        return pd.DataFrame(x_scaled)

    @staticmethod
    def create_result_array(acc_entity):
        array = []
        array1 = []

        # storing in array
        array.append(acc_entity.filename)
        array.append(acc_entity.score)
        array.append(acc_entity.val_score)
        array.append(acc_entity.param)
        array.append(acc_entity.metrics)
        array1.append(array)

        return array1

    @staticmethod
    def create_submission_array(sub_entity):

        array1 = []
        y_pred = sub_entity.predictions
        id_ = sub_entity.id_
        id_2 = sub_entity.id_2
        id_3 = sub_entity.id_3

        for i in range(len(y_pred)):
            array = []
            # storing in array
            if id_:
                array.append(id_[i])

            if id_2:
                array.append(id_2[i])

            if id_3:
                array.append(id_3[i])

            array.append(y_pred[i])

            array1.append(array)

        return array1

    @staticmethod
    def classification_synthesize_data(train, df, label_name):
        os = RandomOverSampler(1.0)  # Ratio decides how much percent of majority minority should have
        smote_x_train, smote_y_train = os.fit_resample(train, df)
        print("LABEL COUNT AFTER SMOTE")
        print(smote_y_train.value_counts(), "\n")

        '''
        # Synthesize minority class data_points using SMOTE
        sm = SMOTE(random_state=42, sampling_strategy='minority')
        smote_x_train, smote_y_train = sm.fit_resample(train, df)
        # Separate into training and test sets
        smote_x_train = pd.DataFrame(smote_x_train, columns = train.columns)
        smote_y_train = pd.DataFrame(smote_y_train, columns = ['Loan_Status'])
        print(smote_y_train,"\n")
        '''

        return smote_x_train, smote_y_train

    @staticmethod
    def synthesize_data(train, df, label_name):
        print("LABEL COUNT BEFORE SMOTE")
        print(df.value_counts(), "\n")
        # Synthesize minority class data points using SMOTE
        sm = SMOTE(random_state=42, sampling_strategy='minority')
        smote_x_train, smote_y_train = train, df
        # Separate into training and test sets
        label_count = len(df.value_counts()) - 1
        for i in range(0, label_count):
            smote_x_train, smote_y_train = sm.fit_resample(smote_x_train, smote_y_train)
            smote_x_train = pd.DataFrame(smote_x_train, columns=train.columns)
            smote_y_train = pd.DataFrame(smote_y_train, columns=[label_name])

        print("LABEL COUNT AFTER SMOTE")
        print(smote_y_train[label_name].value_counts(), "\n")

        '''
        #Randomoversampler
        os=RandomOverSampler(1.0) #Ratio decides how much percent of majority minority should have
        smote_x_train,smote_y_train=os.fit_sample(train,df)
        # Separate into training and test sets
        smote_x_train = pd.DataFrame(smote_x_train, columns = train.columns)
        smote_y_train = pd.DataFrame(smote_y_train, columns = ['Surge_Pricing_Type'])
        smote_x_train,smote_y_train=os.fit_sample(train,df)
        smote_x_train = pd.DataFrame(smote_x_train, columns = train.columns)
        smote_y_train = pd.DataFrame(smote_y_train, columns = ['Surge_Pricing_Type'])
        print (smote_y_train['Surge_Pricing_Type'].value_counts(),"\n")'''

        return smote_x_train, smote_y_train



# IMPORT
import lightgbm as lgb
from aiautomation.mlpackage.Entities import ClasRegModelEntity
from aiautomation.mlpackage.PackageVariable import Variable
from catboost import CatBoostClassifier
from sklearn.ensemble import RandomForestClassifier, GradientBoostingClassifier, AdaBoostClassifier
from sklearn.linear_model import LogisticRegression, RidgeClassifier
from sklearn.naive_bayes import BernoulliNB, GaussianNB
from sklearn.neighbors import KNeighborsClassifier
from sklearn.svm import SVC
from sklearn.tree import DecisionTreeClassifier
from xgboost import XGBClassifier


class Models:

    def __init__(self):
        self.alg_name = {}

    # define different models
    # LOGISTIC REGRESSION
    def log_reg(self):
        # define models and parameters˳
        # TODO max_iteration =1000
        logistic_reg = LogisticRegression(max_iter=1100)

        # define grid search
        solvers = ['newton-cg', 'lbfgs', 'liblinear', 'sag', 'saga']
        penalty = ['l2']
        c_values = [100, 10, 1.0, 0.1, 0.01]

        # define grid search
        grid = dict(solver=solvers, penalty=penalty, C=c_values)

        clas_reg_model_entity = ClasRegModelEntity(logistic_reg, grid, Variable.typeLogReg)
        self.alg_name[logistic_reg] = 'sklearn.linear_model.LogisticRegression'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # RIDGE CLASSIFIER
    def rid_clas(self):
        # define models and parameters
        rid_clas = RidgeClassifier()

        # Whole values
        alpha = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]

        # define grid search
        grid = dict(alpha=alpha)

        clas_reg_model_entity = ClasRegModelEntity(rid_clas, grid, Variable.typeRidge)
        self.alg_name[rid_clas] = 'sklearn.linear_model.RidgeClassifier'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # K NEAREST NEIGHBOUR
    def knn(self):
        # define models and parameters
        knn = KNeighborsClassifier()

        # Whole values
        n_neighbors = list(range(1, 21, 2))
        weights = ['uniform', 'distance']
        metric = ['euclidean', 'manhattan', 'minkowski']

        # define grid search
        grid = dict(n_neighbors=n_neighbors, weights=weights, metric=metric)

        clas_reg_model_entity = ClasRegModelEntity(knn, grid, Variable.typeKnn)
        self.alg_name[knn] = 'sklearn.neighbors.KNeighborsClassifier'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Support Vector Machine (SVM)
    def svm(self):
        # define models and parameters
        svc_classifier = SVC()

        # Whole values
        kernel = ['poly', 'rbf', 'sigmoid']
        c = [100, 10, 1.0, 0.1, 0.01]
        gamma = ['scale', 'auto']

        # define grid search
        grid = dict(kernel=kernel, C=c, gamma=gamma)

        clas_reg_model_entity = ClasRegModelEntity(svc_classifier, grid, Variable.typeSvc)
        self.alg_name[svc_classifier] = 'sklearn.svm.SVC'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # DECISION TREE
    def des_tree(self, max_feature):
        # define models and parameters
        des_tree_classifier = DecisionTreeClassifier()

        # Whole values
        max_depth = [3, None]
        max_features = list(range(1, max_feature, 1))
        min_sample_leaf = list(range(1, 10, 1))
        criterion = ['gini', 'entropy']

        # define grid search
        grid = dict(max_depth=max_depth, max_features=max_features, min_samples_leaf=min_sample_leaf,
                    criterion=criterion)

        clas_reg_model_entity = ClasRegModelEntity(des_tree_classifier, grid, Variable.typeDesTree)
        self.alg_name[des_tree_classifier] = 'sklearn.tree.DecisionTreeClassifier'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # RANDOM FOREST
    def ran_for(self):
        # define models and parameters
        ran_for_classifier = RandomForestClassifier()

        # Whole values
        n_estimators = list(range(100, 1001, 100))
        max_features = ['sqrt', 'log2']

        # define grid search
        grid = dict(n_estimators=n_estimators, max_features=max_features)

        clas_reg_model_entity = ClasRegModelEntity(ran_for_classifier, grid, Variable.typeRanFor)
        self.alg_name[ran_for_classifier] = 'sklearn.ensemble.RandomForestClassifier'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # GAUSSIAN NB
    def gaussian_nb(self):
        # define models and parameters
        gaussian_nb = GaussianNB()
        var_smoothing = [1e-9, 1e-8, 1e-7, 1e-6, 1e-5, 1e-4, 1e-3, 1e-2, 1e-1, 1e-0]

        # define grid search
        grid = dict(var_smoothing=var_smoothing)

        clas_reg_model_entity = ClasRegModelEntity(gaussian_nb, grid, Variable.typeGaussianNB)
        self.alg_name[gaussian_nb] = 'sklearn.naive_bayes.GaussianNB'
        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # BERNOULLI NB
    def bernoulli_nb(self):
        # define models and parameters
        bernoulli_nb = BernoulliNB()
        alpha = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]
        fit_prior = [False, True]

        # define grid search
        grid = dict(alpha=alpha, fit_prior=fit_prior)

        clas_reg_model_entity = ClasRegModelEntity(bernoulli_nb, grid, Variable.typeBerNB)
        self.alg_name[bernoulli_nb] = 'sklearn.naive_bayes.BernoulliNB'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Stochastic Gradient Boosting
    def grad_boost(self):
        # define models and parameters
        gra_boost_class = GradientBoostingClassifier()

        # Whole values
        n_estimators = list(range(100, 1000, 100))
        learning_rate = [0.001, 0.01, 0.1]
        subsample = [0.5, 0.7, 1.0]
        max_depth = [3, 7, 9]

        # define grid search
        grid = dict(learning_rate=learning_rate, n_estimators=n_estimators, subsample=subsample, max_depth=max_depth)

        clas_reg_model_entity = ClasRegModelEntity(gra_boost_class, grid, Variable.typeGradBoost)
        self.alg_name[gra_boost_class] = 'sklearn.ensemble.GradientBoostingClassifier'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Adaptive Boosting
    def ada_boost(self):
        """
        max_depth = [3,None]
        max_features = [1,2,3,4,5,6,7,8,9,10]
        min_sample_leaf = [1,2,3,4,5,6,7,8,9,10]
        criterion = ['gini','entropy']

        des_classifier = DecisionTreeClassifier(max_depth=max_depth,max_features=max_features,
        min_samples_leaf=min_sample_leaf,criterion=criterion) """

        # define models and parameters
        ada_boost_class = AdaBoostClassifier()
        n_estimators = list(range(50, 1001, 50))
        learning_rate = [0.001, 0.01, 0.1]
        algorithm = ['SAMME', 'SAMME.R']

        # define grid search
        grid = dict(learning_rate=learning_rate, n_estimators=n_estimators, algorithm=algorithm)

        clas_reg_model_entity = ClasRegModelEntity(ada_boost_class, grid, Variable.typeAdaBoost)
        self.alg_name[ada_boost_class] = 'sklearn.ensemble.AdaBoostClassifier'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # XGB boost
    def xgb(self):
        # define models and parameters
        xgb_model = XGBClassifier()

        # Whole values
        max_depth = list(range(3, 10, 2))
        min_child_weight = range(1, 6, 2)
        gamma = [float(i / 10.0) for i in range(0, 5)]
        n_estimators = list(range(100, 1000, 100))
        learning_rate = [0.001, 0.01, 0.1]
        subsample = [i / 10.0 for i in range(6, 10)],
        col_sample_by_tree = [i / 10.0 for i in range(6, 10)]
        reg_alpha = [1e-5, 1e-2, 0.1, 1, 100, 0, 0.001, 0.005, 0.01, 0.05]

        # define grid search
        grid = dict(learning_rate=learning_rate, n_estimators=n_estimators, subsample=subsample, max_depth=max_depth,
                    min_child_weight=min_child_weight, gamma=gamma, colsample_bytree=col_sample_by_tree,
                    reg_alpha=reg_alpha)

        clas_reg_model_entity = ClasRegModelEntity(xgb_model, grid, Variable.typeXgb)
        self.alg_name[xgb_model] = 'xgboost.XGBClassifier'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Light GBM
    def light_gbm(self):
        # define models and parameters
        light_gbm = lgb.LGBMClassifier()

        # Whole values
        num_leaves = list(range(50, 1000, 50))
        learning_rate = [0.001, 0.01, 0.1]
        max_depth = list(range(3, 10, 2))
        max_bin = list(range(100, 1000, 100))
        objective = 'binary'

        # define grid search
        grid = dict(learning_rate=learning_rate, num_leaves=num_leaves, max_bin=max_bin, max_depth=max_depth,
                    objective=objective)

        clas_reg_model_entity = ClasRegModelEntity(light_gbm, grid, Variable.typeLGBM)
        self.alg_name[light_gbm] = 'lgb.LGBMClassifier'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Cat Boost GBM
    @staticmethod
    def cat_boost_gbm():
        # define models and parameters
        cat_boost = CatBoostClassifier()

        # Whole values
        depth = [3, 1, 2, 6, 4, 5, 7, 8, 9, 10]
        iterations = [250, 100, 500, 1000]
        learning_rate = [0.03, 0.001, 0.01, 0.1, 0.2, 0.3]
        l2_leaf_reg = [3, 1, 5, 10, 100]
        border_count = [32, 5, 10, 20, 50, 100, 200],
        ctr_border_count = [50, 5, 10, 20, 100, 200],
        thread_count = 4

        # define grid search
        grid = dict(learning_rate=learning_rate, depth=depth, iterations=iterations, l2_leaf_reg=l2_leaf_reg,
                    border_count=border_count, ctr_border_count=ctr_border_count, thread_count=thread_count)

        clas_reg_model_entity = ClasRegModelEntity(cat_boost, grid, Variable.typeCGBM)
        # self.alg_name[catboost] = 'catboost.CatBoostClassifier'

        return clas_reg_model_entity

    def get_all_models(self, max_feature):
        print(max_feature)
        return [self.rid_clas(), self.log_reg(), self.knn(), self.bernoulli_nb(),
                self.gaussian_nb()]  # ,self.des_tree(max_feature), self.svm(), self.ran_for(), self.ada_boost(),
        # self.grad_boost(), self.xgb(), self.light_gbm(), self.cat_boost_gbm()]

    def get_algorithm_name(self):
        return self.alg_name



# IMPORT
import numpy as np
from sklearn.preprocessing import LabelEncoder
from aiautomation.mlpackage.Repository import Repository
from aiautomation.mlpackage.PackageVariable import Variable
from aiautomation.mlpackage.Entities import HyperParamEntity
from aiautomation.mlpackage.multiclassification.MultiClassModel import Models
from sklearn.model_selection import RepeatedStratifiedKFold
from aiautomation.mlpackage.HyperparameterTuning import HyperParameterTuning


class MultiClassifyRepository(Repository):

    def __init__(self):
        super().__init__()
        super().set_type(Variable.typeMultiClass)
        self.labels = []

    def process_data_and_run(self, train, label_name, df=None):
        train, df = super().process_and_get_train_data(train, label_name, df)

        # Fill nan and check
        print("COLUMN NULL CHECK")
        print(train.isnull().sum(), "\n")

        smote_x_train, smote_y_train = super().synthesize_data(train, df, label_name)

        x = np.array(smote_x_train.values.tolist())
        y = np.array(smote_y_train.values.tolist()).ravel()

        y = self.get_transformed_y(y)

        self.start_train(x, y)

    def get_transformed_y(self, y):
        print("Transformed Y start -----")
        if isinstance(y[0], str):
            pass

        le = LabelEncoder()
        le.fit(y)
        le_name_mapping = dict(zip(le.classes_, le.transform(le.classes_)))

        self.labels = [str(i) for i in le.classes_]

        print("Y before transformed -----------")
        print(y)
        print("Dictionary of label -----------")
        print(le_name_mapping)
        y = le.transform(y)
        print("Y after transformed -----------")
        print(y)
        print("LABELS LIST -------")
        print(self.labels)
        print("\n\n")
        return y

    def update_user_scoring_dict(self, user_scoring_dict=None):
        super().update_user_scoring_dict(user_scoring_dict)

    def start_train(self, x, y):
        super().update_xy_data(x, y)
        x_train, x_val, y_train, y_val = super().get_splitted_data()
        self.run_all_models(x_train, x_val, y_train, y_val)

    def concat_dataset_location(self):
        super().concat_to_dataset_location(Variable.typeMultiClass)

    def run_all_models(self, x_train, x_val, y_train, y_val):
        models = Models()
        model_array = models.get_all_models()

        cv = RepeatedStratifiedKFold(n_splits=10, n_repeats=1, random_state=1)
        hp_entity = HyperParamEntity(x_train, y_train, x_val, y_val, models.get_algorithm_name(), cv,
                                     Variable.typeMultiClass, labels=self.labels)

        hyper_parameter_tuning = HyperParameterTuning(hp_entity)

        super().run_all_models(hyper_parameter_tuning, model_array)



# IMPORT

import sys
import csv
import pickle
import os.path
import pandas as pd
from aiautomation.mlpackage.PackageVariable import Variable


class InputOutputStream:

    def __init__(self):
        self.storeDataDirName = ''
        self.update_store_data_dir_name()

    def update_store_data_dir_name(self):
        if Variable.isRunAllFileEnabled:
            self.storeDataDirName = sys.argv[1] + Variable.locationSeparator

    # write the csv files

    @staticmethod
    def write_csv(filename, array, fields):

        # fields = ["Trip_ID","Surge_Pricing_Type"]
        # writing to csv file

        with open(filename, 'w') as csvfile:
            # creating a csv writer object

            csvwriter = csv.writer(csvfile)

            # writing the fields

            csvwriter.writerow(fields)

            # writing the data rows

            csvwriter.writerows(array)

    def write_acc_model(self, array):
        filename = self.storeDataDirName + Variable.modelFileName
        field = ['Filename', 'Best_Score', 'Best_Val_Score', 'Best_Hyper_Parameters', 'Metrics_Name']
        df = self.get_stored_model_data(filename)
        big_array = df.values.tolist()
        big_array.append(array[0])
        df = pd.DataFrame(big_array, columns=field)
        df.to_feather(filename)


    def write_eda_feather(self, file_name):
        filename = self.storeDataDirName + Variable.edaFileName
        field = ['Filename']
        df = self.get_stored_model_data(filename)  
        big_array = df.values.tolist()
        big_array.append([file_name])
        df = pd.DataFrame(big_array, columns=field)
        df.to_feather(filename)


    def get_stored_model_file_name_array(self):
        df = self.get_stored_model_data(self.storeDataDirName + Variable.modelFileName)
        if df.empty:
            return []
        else:
            return df['Filename'].values.tolist()

    def get_stored_eda_file_name_array(self):
        df = self.get_stored_model_data(self.storeDataDirName + Variable.edaFileName)
        if df.empty:
            return []
        else:
            return df['Filename'].values.tolist()        

    @staticmethod
    def get_stored_model_data(filename):
        if os.path.isfile(filename):
            return pd.DataFrame(pd.read_feather(filename))
        else:
            return pd.DataFrame()

    def save_best_model(self, model_name, model):
        self.remove_all_pickle_file()
        self.save_model(model_name, model, self.storeDataDirName + Variable.bestPickleFolderName)

    def save_all_model(self, model_name, folder_path, model):
        pickle_folder_path = self.storeDataDirName + Variable.allPickleFolderName + Variable.locationSeparator \
                           + folder_path
        self.save_model(model_name, model, pickle_folder_path)

    def save_all_visualizer(self, model_name, folder_path, model, ):
        visualizer_folder_path = self.storeDataDirName + Variable.allVisualizerFolderName + Variable.locationSeparator \
                               + folder_path
        self.save_model(model_name, model, visualizer_folder_path)

    @staticmethod
    def save_model(model_name, model, pickle_folder_path):
        pickle_filename = pickle_folder_path + Variable.locationSeparator + model_name + Variable.pickleExtension
        pickle.dump(model, open(pickle_filename, Variable.writeBinary))

    def export_gene(self, tpot, file_name):
        gene_file_name = self.storeDataDirName + Variable.geneFolderName + Variable.locationSeparator + file_name \
                         + Variable.pythonExtension
        tpot.export(gene_file_name)

    def remove_all_pickle_file(self):
        for filename in os.scandir(self.storeDataDirName + Variable.bestPickleFolderName):
            os.remove(filename.path)

    def check_and_create_all_dirs(self):
        self.check_and_create_dir(self.storeDataDirName + Variable.dataFolderName)
        self.check_and_create_dir(self.storeDataDirName + Variable.geneFolderName)
        self.check_and_create_dir(self.storeDataDirName + Variable.allPickleFolderName)
        self.check_and_create_dir(self.storeDataDirName + Variable.bestPickleFolderName)
        self.check_and_create_dir(self.storeDataDirName + Variable.allVisualizerFolderName)
        self.check_and_create_dir(self.storeDataDirName + Variable.edaLocation)

    def create_model_folder(self, folder_path):
        self.create_pickle_folder(folder_path)
        self.create_visualizer_folder(folder_path)

    def create_visualizer_folder(self, folder_path):
        visualizer_folder_path = self.storeDataDirName + Variable.allVisualizerFolderName + Variable.locationSeparator \
                               + folder_path
        self.check_and_create_dir(visualizer_folder_path)

    def create_pickle_folder(self, folder_path):
        pickle_folder_path = self.storeDataDirName + Variable.allPickleFolderName + Variable.locationSeparator \
                           + folder_path
        self.check_and_create_dir(pickle_folder_path)

    @staticmethod
    def check_and_create_dir(directory_name):
        if not os.path.exists(directory_name):
            os.mkdir(directory_name)

    def check_and_save_as_feather(self, actual_folder_path, csv_file_name, feather_file_name):
        filename = actual_folder_path + csv_file_name
        if os.path.isfile(filename):
            df = pd.read_csv(filename)
            feather_folder_path = actual_folder_path + Variable.featherFolderName
            self.check_and_create_dir(feather_folder_path)
            feather_file_path = feather_folder_path + feather_file_name
            if os.path.isfile(feather_file_path):
                return
            df.to_feather(feather_file_path)

    def get_saved_model(self):
        pickle_model_dir = self.storeDataDirName + Variable.bestPickleFolderName
        pickle_filename = pickle_model_dir + Variable.locationSeparator + os.listdir(pickle_model_dir)[0]
        print(pickle_filename)
        pickle_file = open(pickle_filename, Variable.readBinary)
        return pickle.load(pickle_file)

    def is_model_present(self):
        return os.listdir(self.storeDataDirName + Variable.bestPickleFolderName) != []



class HyperParamEntity:

    def __init__(self, x_train=[], y_train=[], x_val=[], y_val=[], alg_name={}, cv=None, model_type="", scoring="",
                 labels=[]):
        self.x_train = x_train
        self.y_train = y_train
        self.x_val = x_val
        self.y_val = y_val
        self.scoring = scoring
        self.cv = cv
        self.model_type = model_type
        self.alg_name = alg_name
        self.labels = labels


class ClasRegModelEntity:
    def __init__(self, alg, grid, model_name):
        self.alg = alg
        self.grid = grid
        self.model_name = model_name


class MultiClasModelEntity:
    def __init__(self, alg, grid, model_name, gene_grid, gene_alg):
        self.alg = alg
        self.grid = grid
        self.model_name = model_name
        self.gene_grid = gene_grid
        self.gene_alg = gene_alg


class AccuracyEntity:

    def __init__(self, file_name, score=0, val_score=0, param="", metrics="", error_msg=""):
        self.filename = file_name
        self.score = score
        self.val_score = val_score
        self.param = param
        self.metrics = metrics
        self.error_msg = error_msg


class SubmissionEntity:
    def __init__(self, predictions=[], id_=[], id_2=[], id_3=[], fields=[], file_name=None):
        self.predictions = predictions
        self.id_ = id_
        self.id_2 = id_2
        self.id_3 = id_3
        self.fields = fields
        self.fileName = file_name



# IMPORT
import numpy as np
from sklearn.model_selection import KFold
from aiautomation.mlpackage.Repository import Repository
from aiautomation.mlpackage.regression.RegressionModel import Models
from aiautomation.mlpackage.PackageVariable import Variable
from aiautomation.mlpackage.Entities import HyperParamEntity
from aiautomation.mlpackage.HyperparameterTuning import HyperParameterTuning


class RegRepository(Repository):

    def __init__(self):
        super().__init__()
        super().set_type(Variable.typeRegress)

    def process_data_and_run(self, train, label_name, df=None):
        train, df = super().process_and_get_train_data(train, label_name, df)

        # Fill nan and check
        print("COLUMN NULL CHECK TRAIN")
        print(train.isnull().sum(), "\n")

        x = np.array(train.values.tolist())
        y = np.array(df.values.tolist())

        print(x.shape)
        print(y.shape, "\n")
        self.start_train(x, y)

    def update_user_scoring_dict(self, user_scoring_dict=None):
        super().update_user_scoring_dict(user_scoring_dict)

    def start_train(self, x, y):
        super().update_xy_data(x, y)
        x_train, x_val, y_train, y_val = super().get_splitted_data()
        self.run_all_models(x_train, y_train, x_val, y_val)

    def concat_dataset_location(self):
        super().concat_to_dataset_location(Variable.typeRegress)

    def run_all_models(self, x_train, y_train, x_val, y_val):
        models = Models()
        model_array = models.get_all_models()

        cv = KFold(n_splits=10, shuffle=True, random_state=100)
        hp_entity = HyperParamEntity(x_train, y_train, x_val, y_val, models.get_algorithm_name(), cv,
                                     Variable.typeRegress)

        hyper_parameter_tuning = HyperParameterTuning(hp_entity)

        super().run_all_models(hyper_parameter_tuning, model_array)



# IMPORT
import numpy as np
import lightgbm as lgb
from sklearn.svm import SVR
from xgboost import XGBRegressor
from catboost import CatBoostRegressor
from aiautomation.mlpackage.Entities import ClasRegModelEntity
from sklearn.tree import DecisionTreeRegressor
from sklearn.neighbors import KNeighborsRegressor
from aiautomation.mlpackage.PackageVariable import Variable
from sklearn.linear_model import LinearRegression, Ridge
from sklearn.ensemble import RandomForestRegressor, GradientBoostingRegressor, AdaBoostRegressor


class Models:

    def __init__(self):
        self.alg_name = {}

    # define different models
    # LOGISTIC REGRESSION
    def lin_reg(self):
        # define models and parameters
        linreg = LinearRegression()
        fit_intercept = [True, False]
        # normalize = [True, False]
        copy_x = [True, False]

        # define grid search
        grid = dict(fit_intercept=fit_intercept, copy_X=copy_x)

        clas_reg_model_entity = ClasRegModelEntity(linreg, grid, Variable.typeLinReg)
        self.alg_name[linreg] = 'sklearn.linear_model.LinearRegression'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # RIDGE CLASSIFIER
    def rid_reg(self):
        # define models and parameters
        rid_reg = Ridge()
        alpha = [0.0001, 0.001, 0.01, 0.1, 1.0]
        # normalize = [True, False]
        fit_intercept = [True, False]
        solver = ['auto', 'svd', 'cholesky', 'lsqr', 'sparse_cg', 'sag', 'saga']

        # define grid search
        grid = dict(alpha=alpha, solver=solver, fit_intercept=fit_intercept)

        clas_reg_model_entity = ClasRegModelEntity(rid_reg, grid, Variable.typeRidge)
        self.alg_name[rid_reg] = 'sklearn.linear_model.Ridge'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # K NEAREST NEIGHBOUR
    def knn(self):
        # define models and parameters
        knn = KNeighborsRegressor()
        n_neighbors = list(range(1, 21, 2))
        weights = ['uniform', 'distance']
        metric = ['euclidean', 'manhattan', 'minkowski']

        # define grid search
        grid = dict(n_neighbors=n_neighbors, weights=weights, metric=metric)

        clas_reg_model_entity = ClasRegModelEntity(knn, grid, Variable.typeKnn)
        self.alg_name[knn] = 'sklearn.neighbors.KNeighborsRegressor'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Support Vector Machine (SVM)
    def svr(self):
        # define models and parameters
        svr = SVR()
        kernel = ['linear', 'poly', 'rbf', 'sigmoid']
        gamma = ['scale', 'auto']
        c = [100, 10, 1.0, 0.1, 0.01]
        # epsilon = [0.1,0.2,0.5,0.3]

        # define grid search
        grid = dict(kernel=kernel, gamma=gamma, C=c)

        clas_reg_model_entity = ClasRegModelEntity(svr, grid, Variable.typeSvc)
        self.alg_name[svr] = 'sklearn.svm.SVR'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # DECISION TREE
    def des_tree(self):
        # define models and parameters
        des_tree_reg = DecisionTreeRegressor()
        criterion = ['mse', 'friedman_mse', 'mae']
        splitter = ['best', 'random']
        max_features = ['auto', 'sqrt', 'log2']
        max_depth = list(range(5, 150, 15))
        min_samples_leaf = list(range(10, 150, 10))
        min_samples_leaf.append(1)
        max_leaf_nodes = list(range(5, 150, 15))
        min_samples_split = list(range(10, 150, 10))

        # define grid search
        grid = dict(criterion=criterion, splitter=splitter, max_features=max_features, max_depth=max_depth,
                    min_samples_leaf=min_samples_leaf,
                    max_leaf_nodes=max_leaf_nodes, min_samples_split=min_samples_split)

        clas_reg_model_entity = ClasRegModelEntity(des_tree_reg, grid, Variable.typeDesTree)
        self.alg_name[des_tree_reg] = 'sklearn.tree.DecisionTreeRegressor'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # RANDOM FOREST
    def ran_for(self):
        # define models and parameters
        ran_for_reg = RandomForestRegressor()
        n_estimators = [int(x) for x in np.linspace(start=100, stop=1200, num=12)]
        max_features = ['auto', 'sqrt', 'log2']
        max_depth = [int(x) for x in np.linspace(5, 30, num=6)]
        max_depth.append(None)
        min_samples_split = [2, 5, 10, 15, 100]
        min_samples_leaf = [1, 2, 5, 10]
        criterion = ['mse', 'mae']

        # define grid search
        grid = dict(n_estimators=n_estimators, max_features=max_features, max_depth=max_depth,
                    min_samples_split=min_samples_split,
                    min_samples_leaf=min_samples_leaf, criterion=criterion)

        clas_reg_model_entity = ClasRegModelEntity(ran_for_reg, grid, Variable.typeRanFor)
        self.alg_name[ran_for_reg] = 'sklearn.ensemble.RandomForestRegressor'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Stochastic Gradient Boosting
    def grad_boost(self):
        # define models and parameters
        gra_boost_reg = GradientBoostingRegressor()
        loss = ['ls', 'lad', 'huber', 'quantile']
        learning_rate = [0.03, 0.001, 0.01, 0.1, 0.2, 0.3]
        n_estimators = list(range(100, 1000, 100))
        criterion = ['friedman_mse', 'mse', 'mae']
        subsample = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]
        # alpha = [0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]
        max_features = ['auto', 'sqrt', 'log2']
        max_depth = list(range(1, 11, 1))

        # define grid search
        grid = dict(loss=loss, learning_rate=learning_rate, n_estimators=n_estimators, criterion=criterion,
                    subsample=subsample,
                    max_features=max_features, max_depth=max_depth)

        clas_reg_model_entity = ClasRegModelEntity(gra_boost_reg, grid, Variable.typeGradBoost)
        self.alg_name[gra_boost_reg] = 'sklearn.ensemble.GradientBoostingRegressor'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Adaptive Boosting
    def ada_boost(self):
        """
        max_depth = [3,None]
        max_features = [1,2,3,4,5,6,7,8,9,10]
        min_sample_leaf = [1,2,3,4,5,6,7,8,9,10]
        criterion = ['gini','entropy']

        des_classifier = DecisionTreeClassifier(max_depth=max_depth,max_features=max_features,
        min_samples_leaf=min_sample_leaf,criterion=criterion) """

        # define models and parameters
        ada_boost_reg = AdaBoostRegressor()
        n_estimators = list(range(50, 1001, 50))
        learning_rate = [0.001, 0.01, 0.1]
        loss = ['linear', 'square', 'exponential']

        # define grid search
        grid = dict(learning_rate=learning_rate, n_estimators=n_estimators, loss=loss)

        clas_reg_model_entity = ClasRegModelEntity(ada_boost_reg, grid, Variable.typeAdaBoost)
        self.alg_name[ada_boost_reg] = 'sklearn.ensemble.AdaBoostRegressor'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # XGB boost
    def xgb(self):
        # define models and parameters
        xgb_model = XGBRegressor()
        max_depth = list(range(3, 10, 2))
        min_child_weight = list(range(1, 6, 2))
        gamma = [float(i / 10.0) for i in range(0, 5)]
        n_estimators = list(range(100, 1000, 100))
        learning_rate = [0.03, 0.001, 0.01, 0.1, 0.2, 0.3]
        subsample = [i / 10.0 for i in range(6, 10)],
        col_sample_by_tree = [i / 10.0 for i in range(6, 10)]
        reg_alpha = [1e-5, 1e-2, 0.1, 1, 100, 0, 0.001, 0.005, 0.01, 0.05]

        # define grid search
        grid = dict(learning_rate=learning_rate, n_estimators=n_estimators, subsample=subsample, max_depth=max_depth,
                    min_child_weight=min_child_weight, gamma=gamma, colsample_bytree=col_sample_by_tree,
                    reg_alpha=reg_alpha)

        clas_reg_model_entity = ClasRegModelEntity(xgb_model, grid, Variable.typeXgb)
        self.alg_name[xgb_model] = 'xgboost.XGBRegressor'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Light GBM
    def light_gbm(self):
        # define models and parameters
        lightgbm = lgb.LGBMRegressor()
        num_leaves = list(range(50, 1000, 50))
        learning_rate = [0.03, 0.001, 0.01, 0.1, 0.2, 0.3]
        max_depth = list(range(3, 10, 2))
        max_bin = list(range(100, 1000, 100))

        # define grid search
        grid = dict(num_leaves=num_leaves, learning_rate=learning_rate, max_depth=max_depth, max_bin=max_bin)

        clas_reg_model_entity = ClasRegModelEntity(lightgbm, grid, Variable.typeLGBM)
        self.alg_name[lightgbm] = 'lgb.LGBMRegressor'

        return clas_reg_model_entity

    # -----------------------------------------------------------------------------
    # Cat Boost GBM
    @staticmethod
    def cat_boost_gbm():
        # define models and parameters
        catboost = CatBoostRegressor()
        depth = list(range(1, 11, 1))
        iterations = [250, 100, 500, 1000]
        learning_rate = [0.03, 0.001, 0.01, 0.1, 0.2, 0.3]
        l2_leaf_reg = [3, 1, 5, 10, 100]
        border_count = [32, 5, 10, 20, 50, 100, 200],
        ctr_border_count = [50, 5, 10, 20, 100, 200],
        thread_count = 4

        # define grid search
        grid = dict(depth=depth, iterations=iterations, learning_rate=learning_rate, l2_leaf_reg=l2_leaf_reg,
                    border_count=border_count, ctr_border_count=ctr_border_count, thread_count=thread_count)

        clas_reg_model_entity = ClasRegModelEntity(catboost, grid, Variable.typeCGBM)
        # self.alg_name[catboost] = 'catboost.CatBoostRegressor'

        return clas_reg_model_entity

    '''#-----------------------------------------------------------------------------
    #HyperParameters

    epoch = 1000
    batch_size = 128
    input_dim = len(X[0])
    hidden_units_1 = 16
    hidden_units_2 = 6
    output_dim = 1
    dropout = 0.4

    def baseline_model():
        model = Sequential()
        model.add(Dense(hidden_units_1,input_dim=input_dim,activation='relu'))
        model.add(Dropout(dropout))
        model.add(Dense(hidden_units_2,activation='relu'))
        model.add(Dropout(dropout))
        #model.add(Dense(hidden_units_3,activation='relu'))
        #model.add(Dropout(dropout))
        model.add(Dense(output_dim,kernel_initializer='normal',activation = 'linear'))
        model.compile(loss='mean_squared_error', optimizer='adam')
        return model

    estimators = []
    estimators.append(('standardize', StandardScaler()))
    estimators.append(('mlp', KerasRegressor(build_fn=baseline_model, epochs=50, batch_size=5, verbose=0)))
    pipeline = Pipeline(estimators)
    kfold = KFold(n_splits=10, random_state=7)
    results = cross_val_score(pipeline, X, Y, cv=kfold, n_jobs=1)
    print("Standardized: %.2f (%.2f) MSE" % (results.mean(), results.std()))

    # compile the keras model
    # simple early stopping
    es = EarlyStopping(monitor='loss', mode='min', verbose=1)

    # fit the keras model to the dataset
    model.fit(X, Y, epochs=epoch, batch_size=batch_size)	

    # evaluate the keras model
    _, accuracy = model.evaluate(X, Y)
    print('Accuracy: %.2f' % (accuracy*100))'''

    def get_all_models(self):
        return [self.rid_reg(), self.lin_reg(),
                self.knn()]  # , self.des_tree(), self.svr()]#, self.ran_for(), self.adaBoost(), self.grad_boost(),
        # self.xgb(), self.light_gbm(), self.cat_boost_gbm()]

    def get_algorithm_name(self):
        return self.alg_name
